# xDevs.com Calibration app for Keithley 2000, v0.3
# https://xdevs.com/fix/kei2400
# https://xdevs.com/fix/hp3458a
import sys
import time
import Gpib
import signal
import numbers
from math import log10
from Adafruit_BME280 import *

dmm_val = 1.0
dmm_temp = 36.6
exttemp = 25.0
rh = 50.0
pascals = 100000.00
hectopascals = 1000.00

class Timeout():
  """Timeout class using ALARM signal"""
  class Timeout(Exception): pass

  def __init__(self, sec):
    self.sec = sec

  def __enter__(self):
    signal.signal(signal.SIGALRM, self.raise_timeout)
    signal.alarm(self.sec)

  def __exit__(self, *args):
    signal.alarm(0) # disable alarm

  def raise_timeout(self, *args):
    raise Timeout.Timeout()

# THB sensor BME280

def get_THP():
    global exttemp
    global rh
    global pascals
    global hectopascals
    exttemp = sensor.read_temperature()
    rh = sensor.read_humidity()
    pascals = sensor.read_pressure()
    hectopascals = pascals / 100

class dmm():
    global dmm_val 
    data = ""
    temp = 36.6
    cnti = 12
    
    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 3458A
        self.inst.write ("PRESET NORM")
        self.inst.write ("OFORMAT ASCII")
        self.inst.write ("FUNC DCV,AUTO")
        self.inst.write ("TARM HOLD")
        self.inst.write ("TRIG LINE")
        self.inst.write ("NPLC 50")
        self.inst.write ("AZERO ON")
        self.inst.write ("NRDGS 1,AUTO")
        self.inst.write ("MEM OFF")
        self.inst.write ("END ALWAYS")
        self.inst.write ("NDIG 9")

    def switch_dcv(self):
        self.inst.write ("PRESET NORM")
        self.inst.write ("NPLC 50")
        self.inst.write ("AZERO ON")
        self.inst.write ("NRDGS 1,AUTO")
        self.inst.write ("OFORMAT ASCII")
	self.inst.write ("FUNC DCV,AUTO")
        self.inst.write ("NDIG 9")


    def switch_dci(self):
        self.inst.write ("PRESET NORM")
        self.inst.write ("NPLC 50")
        self.inst.write ("AZERO ON")
        self.inst.write ("NRDGS 1,AUTO")
        self.inst.write ("OFORMAT ASCII")
	self.inst.write ("FUNC DCI,AUTO")
        self.inst.write ("NDIG 9")

    def switch_ohmf(self):
        self.inst.write ("PRESET NORM")
        self.inst.write ("NPLC 50")
        self.inst.write ("AZERO ON")
        self.inst.write ("NRDGS 1,AUTO")
        self.inst.write ("OFORMAT ASCII")
	self.inst.write ("FUNC OHMF,AUTO")
        self.inst.write ("NDIG 9")
        
    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        
        try:
            with Timeout(20):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception
        
    def get_temp(self):
        global dmm_temp
        print("Reading DMM temp")
        self.inst.write("TARM SGL,1")
        self.temp_status_flag,temp = self.read_data("TEMP?")
        if (self.temp_status_flag):
            self.temp = temp
	    dmm_temp = temp
        return self.temp
        
    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        global dmm_val
        self.status_flag,data = self.read_data("TARM SGL,1")
        if (self.status_flag):
            self.data = data
            dmm_val = float(data)
            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
        return self.data
        
    def exec_acal(self):
        sys.stdout.write ("\033[1;35mACAL ALL procedure start, please wait 12 minutes ")
        self.inst.write("ACAL ALL")
        for cnt in range(0 ,48):        # wait 720 seconds, print dot every 15s
            sys.stdout.write ("\033[0;43m*")
            time.sleep(25)
        print("\r\nACAL procedure done...\033[1;39m")
    
    def exec_idn(self):
        self.inst.write ("END ALWAYS")
        self.inst.write ("ID?")
	dat = self.inst.read()
	tstr = dat.split()
        if (tstr[0] == "HP3458A"):
            sys.stdout.write ("\r\n\033[1;32m%s detected...\033[1;39m" % tstr[0])
        else:
            sys.stdout.write ("\r\n\033[1;31mNo DMM present, exiting!\033[1;39m")
            quit()

    def get_data_status(self):
        return self.status_flag

class mfc():
    global dmm_val 
    data = ""
    temp = 36.6
    cnti = 12
    
    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup MFC
	print ("No init done here\r\n")
        #self.inst.write ("E1\n")

    def dcv(self,voltage):
	print ("Setting MFC to %f\r\n" % float(voltage))
        self.inst.write ("%f\n" % voltage)

    def ohmf(self,res):
	res_mfc_id = [1, 2, 3, 4, 5, 6, 7]
	res_mfc = ["10 Ohm","100 Ohm","1 KOhm", "10 KOhm", "100 KOhm", "1 MOhm", "10 MOhm"]
	
	if ((res > 0) and (res < 8)):
	    print ("Setting MFC to %d R\r\n" % res_mfc[res] )
    	    self.inst.write ("O%c\n" % res_mfc_id[res])
	else:
	    print ("Incorrect MFC ohm setting, valid from 1 to 7\r\n")
        
    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        
        try:
            with Timeout(20):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception
        
    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        global dmm_val
        self.status_flag,data = self.read_data("TARM SGL,1")
        if (self.status_flag):
            self.data = data
            dmm_val = float(data)
            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
        return self.data
        
    def exec_idn(self):
        self.inst.write ("END ALWAYS")
        self.inst.write ("ID?")
	dat = self.inst.read()
	tstr = dat.split()
        if (tstr[0] == "HP3458A"):
            sys.stdout.write ("\r\n\033[1;32m%s detected...\033[1;39m" % tstr[0])
        else:
            sys.stdout.write ("\r\n\033[1;31mNo DMM present, exiting!\033[1;39m")
            quit()

    def get_data_status(self):
        return self.status_flag


with open('calk2000_ver3.txt','wb') as b:
    kei2000 = Gpib.Gpib(0,20)
    kei2000.clear()

    ref_res_meas = [10.000196523333335, 99.96883466666667, 999.6872704000001, 9999.419994000002, 99997.92083333334, 1000078.4619999999, 10010462.023333332]
    ref_dci_set = [0.09998575900000002, 0.999995969, 9.999901119999999, 100.00129909, 1.0000781718]
    ref_dci_meas = [9.999802283999999e-05, 0.0009999978064666667, 0.009999997660666666, 0.09999987479, 0.9999943627333333]
    cal_dcv_set = [-0.2, 0, 0.2, 0, -2, 0, 2, 0, -20, 0, 20, 0, -200, 0, 200, 0]
    cal_dcv_rng = [0.2, 0.2, 0.2, 0.2, 2, 2, 2, 2, 20, 20, 20, 20, 200, 200, 200, 200]
    cal_dcv_str = ["-0.2 m"," -Zero"," 0.2 m"," +Zero","   -2 "," -Zero","    2 "," -Zero","  -20 "," +Zero","   20 "," -Zero","\033[0;41m -200 \033[0;39m"," +Zero","\033[0;41m  200 \033[0;39m"," +Zero"]
    cal_dci_set = [-1E-6, 0, 1E-6, 0, -10E-6, 0, 10E-6, 0, -100E-6, 0, 100E-6, 0, -1E-3, 0, 1E-3, 0, -10E-3, 0, 10E-3, 0, -100E-3, 0, 100E-3, 0, -1, 0, 1, 0]
    cal_dci_rng = [1E-6, 1E-6, 1E-6, 1E-6, 10E-6, 10E-6, 10E-6, 10E-6, 100E-6, 100E-6, 100E-6, 100E-6, 1E-3, 1E-3, 1E-3, 1E-3, 10E-3, 10E-3, 10E-3, 10E-3, 100E-3, 100E-3, 100E-3, 100E-3, 1, 1, 1, 1]
    cal_dci_str = ["-1 u","-Zero", " 1 u", "+Zero", " -10 u", "-Zero", " 10 u", "+Zero", "-100 u", "-Zero", " 100 u", "+Zero", "-1 m", "-Zero", " 1 m", "+Zero", " -10 m", "-Zero", " 10 m", "+Zero", "-100 m", "-Zero", " 100 m", "+Zero", " -1 ", "-Zero", " 1 ", "+Zero"]
    test_dcv_set = [0, -0.002, 0.002, -0.02, 0.02, -0.2, 0.2, -1, 1, -2, 2, -10, 10, -15, 15, -20, 20, -100, 100, -200, 200, 0]
    test_dcv_rng = [0.2,  0.2,   0.2,   0.2,  0.2,  0.2, 0.2,  2, 2, 20,20,  20, 20,  20, 20,  20, 20, 200, 200, 200, 200, 20]
    test_dci_set = [0, -1E-9, 1E-9, -10E-9, 10E-9, -100E-9, 100E-9, -1E-6, 1E-6, -10E-6, 10E-6, -100E-6, 100E-6, -1E-3, 1E-3, -4E-3, 4E-3, -10E-3, 10E-3, -20E-3, 20E-3, -100E-3, 100E-3, -500E-3, 500E-3, 1, -1, 0]
    test_dci_rng = [1E-6, 1E-6, 1E-6, 1E-6, 1E-6, 1E-6,  1E-6, 1E-6, 1E-6, 10E-6, 10E-6, 100E-6, 100E-6, 1E-3, 1E-3, 10E-3, 10E-3, 10E-3, 10E-3, 100E-3, 100E-3, 100E-3, 100E-3, 1, 1, 1, 1, 1E-3]
    test_dci_str = ["Zero  ","  -1 n", "   1 n", " -10 n", "  10 n", "-100 n", " 100 n", "  -1 u", "   1 u", " -10 u", "  10 u", "-100 u", " 100 u", "  -1 m", "   1 m", "  -4 m", "   4 m", " -10 m", "  10 m", " -20 m", "  20 m", "-100 m", " 100 m", "-500 m", " 500 m", "  -1  ", "   1  ", "Zero  "]
    kei2000_idn = "KEITHLEY INSTRUMENTS INC."
    kei2000_mdl = "MODEL 2000"
    orig_val = []
    kei_error = 0
    plc = 60 #60 Hz
    cnt = 0
    samples = 0
    nplc = 0
    val = 3.423
    meas = 0.0
    temp = 36.6
    
    for cnt in range(30):
        orig_val.append(0)
        
    sys.stdout.write("\033[0;36m Keithley 2000 calibration tool \r\n  Using NI GPIB adapter with next instruments config: \r\n * GPIB 20 : Keithley 2000 DMM \r\n * GPIB 3 : HP 3458A 8.5-digit DMM \r\n * GPIB 21 : Time 9823 MFC \r\n ! Do not swap terminals during calibration, high voltage may be present !\033[0;39m\r\n")
    
    kei2000.write ("*IDN?")
    idn_str = kei2000.read()
    print idn_str
    idmfg = idn_str.split(",")
    if (kei2000_idn == idmfg[0]):
        if (kei2000_mdl == idmfg[1]):
            sys.stdout.write("\033[0;32mKeithley Model 2000 - detected, S/N %s, Version: %s\033[0;39m\r\n" % (idmfg[2], idmfg[3]))
	    b.write("\r\nKeithley Model 2000 - detected, S/N %s, Version: %s\r\n" % (idmfg[2], idmfg[3]))
        else:
            print ("\033[0;31mIncorrect Model! Check GPIB address. Calibration cancelled.\033[0;39m")    
            quit()
    else:
        sys.stdout.write ("\033[0;31mNo Keithley instrument detected. Check GPIB address. Calibration cancelled.\033[0;39m")
        quit()
    
    sensor = BME280(mode=BME280_OSAMPLE_8)
    get_THP() # Read THP

    print ("Environment temperature = %2.2f C, humidity = %4.2f RH, pressure = %6.4f hPa" % (exttemp, rh, hectopascals))
    b.write ("Environment temperature = %2.2f C, humidity = %4.2f RH, pressure = %6.4f hPa\r\n" % (exttemp, rh, hectopascals))

    kei2000.write (":SYST:PRES;*CLS")
    kei2000.write ("*ESE 1;*SRE 32")
    
    #kei2000.write (":CAL:PROT:LOCK")
    #kei2000.write (":CAL:PROT:CODE 'KI002000'")
    
    dmm = dmm(3,dmm_val,"3458A") # GPIB 3
    dmm.exec_idn()
    dmm.get_temp()
    
    print ("\033[0;33mHP3458A TEMP = %2.1f C\033[0;33m" % dmm_temp)
    b.write ("HP 3458A detected\r\n TEMP? = %2.1f C\r\n" % dmm_temp)

    mfc = mfc(21,dmm_val,"3458A") # GPIB 3
    mfc.inst.write("R4\n") # set range 10.0
    mfc.dcv(0.000000) # output zero DCV

    print ("\033[1;31mConnect 3458A DMM volts input to Model 2000 INPUT/OUTPUT jacks and to MFC.\033[1;39m")
    raw_input("\033[1;33mPress Enter to continue with DCV test...\033[1;39m")
    
    b.write(" Ambient temp = +26.9C\r\n")

    #dmm.exec_acal()
    #b.write("ACAL DONE\r\n")

    kei2000.write (":SENS:FUNC 'VOLT:DC'")
    kei2000.write (":SENS:VOLT:DC:DIG 7;NPLC 10;AVER:COUN 5; TCON REP")
    kei2000.write (":SENS:VOLT:DC:AVER:STAT ON")
    kei2000.write (":SOUR:VOLT:DC:RANG 10")
    kei2000.write (":TRIG:COUN 1")
    kei2000.write (":TRIG:SOUR IMM")
    kei2000.write (":FORM:ELEM READ")
    
    mfc.dcv(0.000000)
    dmm.switch_dcv()
    dmm.inst.write("RANGE 10")
    print ref_dci_meas
    print ref_dci_set
    print ref_res_meas

    sys.stdout.write("-W- Start K2000 A20 calibration...\r\n\r\n")
    
    kei2000.write (":CAL:PROT:COUNT?")
    print ("Calibrations count")
    cal_cnt = kei2000.read()
    print cal_cnt
    b.write ("Calibrations count = %s\r\n" % cal_cnt)
    kei2000.write (":CAL:PROT:DATE?")
    cal_date = kei2000.read()
    print ("Last calibration date = %s" % cal_date)
    b.write ("Calibration last date = %s\r\n" % cal_date)
    
    kei2000.write (":CAL:PROT:CODE 'KI002000'")
    b.write (":CAL:PROT:CODE 'KI002000'\r\n")
    kei2000.write (":CAL:PROT:INIT")
    kei2000.write (":CAL:PROT:INIT")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    print ("\033[1;31mConnect 4W short plug on K2000 DUT \033[1;39m")
    raw_input("\033[1;33mPress Enter to continue with SHORT calibration...\033[1;39m")

    kei2000.write (":CAL:PROT:DC:STEP1;*OPC")

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)

    raw_input("\033[1;33mPress Enter to continue with OPEN calibration...\033[1;39m")
    kei2000.write (":CAL:PROT:DC:STEP2;*OPC")

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)

    raw_input("\033[1;33mPress Enter to continue with 10VDC calibration...\033[1;39m")
    mfc.inst.write("R4\n") # set range 10.0
    mfc.dcv(10.000000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    time.sleep(4)
    dmm.get_data()
    mfc_delta = (dmm_val) - 10.00000
    print ("CAL MFC error = %5.3f" % (((dmm_val / 10.0) - 1) * 1E6) )
    b.write ("CAL MFC error = %5.3f ppm \r\n" % (((dmm_val / 10.0) - 1) * 1E6) )
    mfc.dcv(10.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))
    kei2000.write (":CAL:PROT:DC:STEP3 %f;*OPC" % refv)
    b.write (":CAL:PROT:DC:STEP3 %f\r\n" % refv)

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mPress Enter to continue with -10VDC calibration...\033[1;39m")
    
    dmm.inst.write("RANGE 10")
    mfc.inst.write("R4\n") # set range 10.0
    mfc.dcv(-10.000000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    time.sleep(4)
    dmm.get_data()
    mfc_delta = (dmm_val) + 10.00000
    print ("CAL MFC error = %5.3f" % (((dmm_val / -10.0) - 1) * 1E6) )
    b.write ("CAL MFC error = %5.3f ppm \r\n" % (((dmm_val / -10.0) - 1) * 1E6) )
    mfc.dcv((-10.0000) - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))
    kei2000.write (":CAL:PROT:DC:STEP4 %f;*OPC" % refv)
    b.write (":CAL:PROT:DC:STEP4 %f\r\n" % refv)

    raw_input("Press Enter to check for error")
    mfc.dcv(0.0)
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)

    raw_input("\033[1;33mPress Enter to continue with 100VDC calibration...\033[1;39m")
    
    dmm.inst.write("RANGE 100")
    mfc.inst.write("R5\n") # set range 10.0
    mfc.dcv(100.000000)
    val = 0.0

    time.sleep(8)
    dmm.get_data()
    time.sleep(8)
    dmm.get_data()
    mfc_delta = (dmm_val) - 100.00000
    print ("CAL MFC error = %5.3f" % (((dmm_val / 100.0) - 1) * 1E6) )
    b.write ("CAL MFC error = %5.3f ppm \r\n" % (((dmm_val / 100.0) - 1) * 1E6) )
    mfc.dcv((100.0000) + mfc_delta)
    time.sleep(4)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))
    kei2000.write (":CAL:PROT:DC:STEP5 %f" % refv)
    b.write (":CAL:PROT:DC:STEP5 %f\r\n" % refv)

    raw_input("Press Enter to check for error")
    mfc.inst.write("R3\n") # set range 10.0
    mfc.dcv(0.000000)
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)

    raw_input("\033[1;33mConnect 4W terminal to K2000 and press Enter to continue with 1 kOhm calibration...\033[1;39m")
    
    mfc.inst.write("R3\n") # set range 10.0
    mfc.dcv(0.000000)
    time.sleep(0.5)
    mfc.inst.write("O3\n") # set range 10.0
    time.sleep(4)
    kei2000.write (":CAL:PROT:DC:STEP6 %f;*OPC" % ref_res_meas[2])
    b.write (":CAL:PROT:DC:STEP6 %f\r\n" % ref_res_meas[2])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect 4W terminal to K2000 and press Enter to continue with 10 kOhm calibration...\033[1;39m")
    
    time.sleep(0.5)
    mfc.inst.write("O4\n") # set range 10.0
    time.sleep(4)
    kei2000.write (":CAL:PROT:DC:STEP7 %f;*OPC" % ref_res_meas[3])
    b.write (":CAL:PROT:DC:STEP7 %f\r\n" % ref_res_meas[3])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect 4W terminal to K2000 and press Enter to continue with 100 kOhm calibration...\033[1;39m")
    
    time.sleep(0.5)
    mfc.inst.write("O5\n") # set range 10.0
    time.sleep(4)
    kei2000.write (":CAL:PROT:DC:STEP8 %f;*OPC" % ref_res_meas[4])
    b.write (":CAL:PROT:DC:STEP8 %f\r\n" % ref_res_meas[4])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect 4W terminal to K2000 and press Enter to continue with 1 MOhm calibration...\033[1;39m")
    
    time.sleep(0.5)
    mfc.inst.write("O6\n") # set range 10.0
    time.sleep(4)
    kei2000.write (":CAL:PROT:DC:STEP9 %f;*OPC" % ref_res_meas[5])
    b.write (":CAL:PROT:DC:STEP9 %f\r\n" % ref_res_meas[5])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect 4W terminal to K2000 and press Enter to continue with 10 MOhm calibration...\033[1;39m")
    
    time.sleep(0.5)
    mfc.inst.write("O7\n") # set range 10.0
    time.sleep(4)
    kei2000.write (":CAL:PROT:DC:STEP10 %f;*OPC" % ref_res_meas[6])
    b.write (":CAL:PROT:DC:STEP10 %f\r\n" % ref_res_meas[6])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect DCI output to K2000 and press Enter to continue with 10 mADC calibration...\033[1;39m")
    
    mfc.inst.write("R9\n") # set range 2mA
    mfc.dcv(ref_dci_set[2])
    time.sleep(3)
    kei2000.write (":CAL:PROT:DC:STEP11 %f;*OPC" % ref_dci_meas[2])
    b.write (":CAL:PROT:DC:STEP11 %f\r\n" % ref_dci_meas[2])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect DCI output to K2000 and press Enter to continue with 100 mADC calibration...\033[1;39m")
    
    mfc.inst.write("R10\n") # set range 100mA
    mfc.dcv(ref_dci_set[3])
    time.sleep(3)
    kei2000.write (":CAL:PROT:DC:STEP12 %f;*OPC" % ref_dci_meas[3])
    b.write (":CAL:PROT:DC:STEP12 %f\r\n" % ref_dci_meas[3])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    raw_input("\033[1;33mConnect DCI output to K2000 and press Enter to continue with 100 mADC calibration...\033[1;39m")
    
    mfc.inst.write("R11\n") # set range 100mA
    mfc.dcv(ref_dci_set[4])
    time.sleep(3)
    kei2000.write (":CAL:PROT:DC:STEP13 %f;*OPC" % ref_dci_meas[4])
    b.write (":CAL:PROT:DC:STEP13 %f\r\n" % ref_dci_meas[4])

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)
    
    sys.stdout.write("-W- End K2000 A20 calibration...\r\n\r\n")
    raw_input("\033[1;33mSave all calibration data? Press Enter to save, or Ctrl-C to abort..\033[1;39m")

    kei2000.write (":CAL:PROT:DATE 2016,07,27")
    kei2000.write (":CAL:PROT:NDUE 2017,07,27")
    kei2000.write (":CAL:PROT:SAVE")
    kei2000.write (":CAL:PROT:LOCK")
    b.write ("K2000 CAL complete \r\n")
    print ("K2000 CAL done")

    raw_input("Press Enter to check for error")
    kei2000.write (":SYST:ERR?")
    errst = kei2000.read()
    b.write ("Error status =  %s\r\n" % errst)
    print ("Error status =  %s" % errst)

    raw_input("\033[1;33mRetest DCV? Press Enter to save, or Ctrl-C to abort..\033[1;39m")
    print ("  -i- Test 0mV VDC \r\n")
    
    dmm.inst.write("RANGE 0.1")
    kei2000.write (":SOUR:VOLT:DC:RANG 0.1")
    mfc.inst.write("R3\n") # set range 10.0
    mfc.dcv(0.00000)
    val = 0.0

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 10mV VDC \r\n")
    
    dmm.inst.write("RANGE 0.1")
    kei2000.write (":SOUR:VOLT:DC:RANG 0.1")
    mfc.inst.write("R1\n") # set range 10.0
    mfc.dcv(10.00000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    dmm.get_data()
    mfc_delta = (dmm_val * 1E3) - 10.00000
    print ("MFC error = %5.3f" % (((dmm_val / 0.01) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 0.01) - 1) * 1E6) )
    mfc.dcv(10.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 100mV VDC \r\n")
    
    dmm.inst.write("RANGE 0.1")
    kei2000.write (":SOUR:VOLT:DC:RANG 0.1")
    mfc.inst.write("R2\n") # set range 10.0
    mfc.dcv(100.00000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    dmm.get_data()
    mfc_delta = (dmm_val * 1E3) - 100.00000
    print ("MFC error = %5.3f" % (((dmm_val / 0.1) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 0.1) - 1) * 1E6) )
    mfc.dcv(100.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 1000mV VDC \r\n")
    
    dmm.inst.write("RANGE 1")
    kei2000.write (":SOUR:VOLT:DC:RANG 1")
    mfc.inst.write("R3\n") # set range 10.0
    mfc.dcv(1.000000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    dmm.get_data()
    mfc_delta = (dmm_val) - 1.00000
    print ("MFC error = %5.3f" % (((dmm_val / 1.0) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 1.0) - 1) * 1E6) )
    mfc.dcv(1.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 10V VDC \r\n")
    
    dmm.inst.write("RANGE 10")
    kei2000.write (":SOUR:VOLT:DC:RANG 10")
    mfc.inst.write("R4\n") # set range 10.0
    mfc.dcv(10.000000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    dmm.get_data()
    mfc_delta = (dmm_val) - 10.00000
    print ("MFC error = %5.3f" % (((dmm_val / 10.0) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 10.0) - 1) * 1E6) )
    mfc.dcv(10.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 100V VDC \r\n")

    mfc.dcv(0.00000)
    
    dmm.inst.write("RANGE 100")
    kei2000.write (":SOUR:VOLT:DC:RANG 100")

    print ("\033[1;31mConnect High voltage terminals at MFC. Warning! Avoid contact with terminals! \033[1;39m")
    raw_input("\033[1;33mPress Enter to continue with HVDC test...\033[1;39m")

    mfc.inst.write("R5\n") # set range 10.0
    mfc.dcv(100.000000)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    time.sleep(4)
    dmm.get_data()
    mfc_delta = (dmm_val) - 100.00000
    print ("MFC error = %5.3f" % (((dmm_val / 100.0) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 100.0) - 1) * 1E6) )
    mfc.dcv(100.0000 - mfc_delta)
    time.sleep(1)
    
    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    print ("  -i- Test 1000V VDC \r\n")
    
    dmm.inst.write("RANGE 1000")
    kei2000.write (":SOUR:VOLT:DC:RANG 1000")

    print ("\033[1;31m Warning! Avoid contact with terminals! \033[1;39m")
    raw_input("\033[1;33mPress Enter to continue with HVDC test...\033[1;39m")

    mfc.inst.write("R6\n") # set range 10.0
    mfc.dcv(1000.000000)
    time.sleep(3)
    val = 0.0

    time.sleep(4)
    dmm.get_data()
    time.sleep(4)
    dmm.get_data()
    mfc_delta = (dmm_val) - 1000.00000
    print ("MFC error = %5.3f" % (((dmm_val / 1000.0) - 1) * 1E6) )
    b.write ("MFC error = %5.3f ppm \r\n" % (((dmm_val / 1000.0) - 1) * 1E6) )
    mfc.dcv(1000.0000 - mfc_delta)
    time.sleep(1)

    for samples in range (0, 6):
        dmm.get_data()
        if (samples > 2):
            #print ("Test REF DCV %d : %.9E DCV" % (samples, dmm_val))
	    val += dmm_val
    refv = val / 3

    print ("Measured REF DCV %d : %.9E VDC" % (samples, refv))

    dval = 0
    samples = 0

    for samples in range (0, 6):
	kei2000.write(":MEAS:VOLT:DC?")
    	val = float(kei2000.read())
        if (samples > 2):
            #print ("Test 2000 DCV %d : %.9E DCV" % (samples, val))
	    dval += val
    dval = dval / 3

    print ("Measured 2000 DCV %d : %.9E VDC" % (samples, dval))
    print ("Delta REF/DUT = %.9E , %5.3f ppm" % ( (refv - dval), (((refv / dval) - 1) * 1E6) ) )
    b.write ("HP 3458A DCV %.9E VDC, K2000 DUT %.9E VDC, Delta %5.3f ppm\r\n" % (refv, dval, (((refv / dval) - 1) * 1E6) ) )

    mfc.dcv(0.00000)
    mfc.inst.write("R4\n") # set range 10.0
    
    quit()
